# keepalive http connection
import httplib, socket, urllib2
from urllib import addinfourl

import sys
def debug(msg, errstream=sys.stderr) :
    if __debug__ :
        print >>errstream, "DEBUG: " + msg

class PooledHTTPConnection :
    _hostmap = {}

    def __init__(self) :
        pass
    
    def connection(self, req) :
        proto = req.get_type()
        assert proto in ('http', 'https')
        netloc = self._netloc(req.get_host(), proto)
        if self._hostmap.has_key((proto, netloc)) :
            con = self._hostmap[(proto, netloc)]
            debug('PooledHTTPCon : reuse hostmap : %s/%s' % (netloc, str(con)))
        else :
            if proto == 'http' :
                con = httplib.HTTPConnection(netloc)
            else :
                con = httplib.HTTPSConnection(netloc)
            self._add_hostmap((proto, netloc), con)
            debug('PooledHTTPCon : add hostmap : (%s, %s)/%s' \
                  % (proto, netloc, str(con)))

        return con

    def _netloc(self, host, proto) :
        try :
            host, port = host.split(':')
        except ValueError :
            if proto == 'http' :
                port = httplib.HTTP_PORT
            else :
                port = httplib.HTTPS_PORT
            
        netloc = '%s:%d' % (host, port)
        return netloc
    
    def _add_hostmap(self, key, con) :
        self._hostmap[key] = con

pool = PooledHTTPConnection()

def PooledBaseHTTPHandler_do_open(self, req):
    """ derived from urllib2.AbstractHTTPHandler.do_open() """

    host = req.get_host()
    if not host:
        raise urllib2.URLError('no host given')

    h = pool.connection(req)
    h.set_debuglevel(self._debuglevel)

    headers = dict(req.headers)
    headers.update(req.unredirected_hdrs)
    
    headers["Connection"] = "keep-alive"
    request_ok = False
    retry = 0
    while not request_ok :
        try:
            h.request(req.get_method(), req.get_selector(), req.data, headers)
            r = h.getresponse()
        except (httplib.BadStatusLine, httplib.CannotSendRequest) :
            if retry > 5 :
                raise
            else :
                h.close()
                h.connect()
            retry += 1
            debug('retry : %d' % retry)
        except socket.error, err: # XXX what error?
            raise urllib2.URLError(err)
        else :
            request_ok = True

    r.recv = r.read
    fp = socket._fileobject(r, 'rb', -1)

    resp = addinfourl(fp, r.msg, req.get_full_url())
    resp.code = r.status
    resp.msg = r.reason
    return resp

class PooledHTTPHandler(urllib2.HTTPHandler) :
    handler_order = 300
    def http_open(self, req) :
        return PooledBaseHTTPHandler_do_open(self, req)
class PooledHTTPSHandler(urllib2.HTTPSHandler) :
    handler_order = 300
    def https_open(self, req) :
        return PooledBaseHTTPHandler_do_open(self, req)

def keepalive_opener() :
    kah = PooledHTTPHandler()
    kahs = PooledHTTPSHandler()
    return urllib2.build_opener(kah, kahs)
